/*
 *	caloopback.c
 *
 *	Copyright 2006
 *
 *	Video Loopback Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2000
 *	Additional copyright by the contributing authors in the
 *	change history below, 2000-2005
 *
 *	Published under the GNU Public License.
 *
 *	This driver creates a virtual 'ca0' device for dvb drivers
 *	and forwards the relevant ioctls to /proc/ca_emu_<num>
 *	(<num> == adapter number).
 *
 *	This code is based off the video-loopback driver
 *	The project is a secondary project for the project "motion" found at
 *	http://motion.sourceforge.net/ and
 *	http://www.lavrsen.dk/twiki/bin/view/Motion/WebHome
 *	and with the vloopback stored at
 *	http://www.lavrsen.dk/twiki/bin/view/Motion/VideoFourLinuxLoopbackDevice
 *
 */


#define CALOOPBACK_VERSION "0.0.1"

/* Include files common to 2.4 and 2.6 versions */
#include <linux/version.h>	/* >= 2.6.14 LINUX_VERSION_CODE */ 
#include <linux/config.h>	/* needed to get LINUX_VERSION_CODE >= 2.6.13 */
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pagemap.h>
#include <linux/dvb/ca.h>
#include <linux/vmalloc.h>
#include <linux/wait.h>
#include <linux/proc_fs.h>

#include "dvbdev.h"

#define info(format, arg...) printk(KERN_INFO __FILE__ ": " format "\n" "", ## arg)


struct vloopback_pipe {
	struct dvb_adapter *adapter;
 	struct dvb_device *dvbdev;
	struct proc_dir_entry *pe;
	struct semaphore lock;
	wait_queue_head_t wait;
	unsigned char *ioctldata;
	int data_ready;
};

#define MAX_PIPES 8

static int devnum[MAX_PIPES];
static int devnum_cnt = 0;
static int lb_count = 0;
static struct vloopback_pipe *loops[MAX_PIPES];

static int create_pipe(int devn, int idx);

static int vloopback_open(struct inode *inode, struct file *f)
{	
	struct dvb_device *dvbdev;
	struct vloopback_pipe *vloop;
	int err;
	dvbdev  = (struct dvb_device *) f->private_data;
	if (dvbdev == NULL) {
		printk("Failed to open device\n");
		return -EFAULT;
	}
	vloop = (struct vloopback_pipe *)dvbdev->priv;
	if (vloop == NULL) {
		printk("Failed to find private data during open\n");
		return -EFAULT;
	}
	err = dvb_generic_open(inode, f);
	if (err < 0)
		return err;
	vloop->ioctldata[0] = 0;
	vloop->data_ready = 0;
	return 0;
}

static ssize_t vloopback_write(struct file *f, const char *buf,
		size_t count, loff_t *offset)
{
	printk("Write not supported on CA device\n");
	return -EFAULT;
}

static ssize_t vloopback_read (struct file * f, char * buf, size_t count, loff_t *offset)
{
	printk("Read not supported on CA device\n");
	return -EFAULT;
}

static int vloopback_ioctl(struct inode *inode, struct file *file,
	unsigned int cmd, void *parg)
{
	struct dvb_device *dvbdev = (struct dvb_device *) file->private_data;
	struct vloopback_pipe *vloop = (struct vloopback_pipe *)dvbdev->priv;
	int size = _IOC_SIZE(cmd);
	int offset;

	if (cmd == CA_GET_CAP) {
		ca_caps_t cap;
		info("Sending CA_GET_CAP\n");
		cap.slot_num = 2;
		cap.slot_type = 10;
                cap.descr_num = 16;
                cap.descr_type = CA_ECD;
                memcpy(parg, &cap, sizeof(cap));
		return 0;
	}

	cmd &= 0xff;
	info("Called ioctl with cmd %d", cmd);

	if (size + sizeof(cmd) > 100)
		return -EFAULT;

	if (down_interruptible(&vloop->lock))
		return -ERESTARTSYS;

	offset = vloop->ioctldata[0];
	if (offset == 10)  {
		memmove(vloop->ioctldata+1, vloop->ioctldata+101, 900);
		offset--;
	}

	memcpy(vloop->ioctldata+1+(100*offset), &cmd, sizeof(cmd));
	memcpy(vloop->ioctldata+1+(100*offset)+sizeof(cmd), (char *)parg, size);
	offset++;
	vloop->ioctldata[0] = offset;
	vloop->data_ready = 1;
	up(&vloop->lock);
	wake_up_interruptible(&vloop->wait);
	info("Forwarded ioctl (%d + %d)", sizeof(cmd), size);

	return 0;
}

static struct file_operations dvb_ca_fops = {
	.owner		= THIS_MODULE,
	.open		= vloopback_open,
	.release	= dvb_generic_release,
	.read		= vloopback_read,
	.write		= vloopback_write,
	.ioctl		= dvb_generic_ioctl,
};

static struct dvb_device dvbdev_ca = {
        .priv = NULL,
        .users = 1,
        .readers = 1,
        .writers = 1,
        .fops = &dvb_ca_fops,
	.kernel_ioctl  = vloopback_ioctl,
};

static struct list_head *dvb_adapter_list = NULL;

static struct dvb_adapter * vloopback_dup_adapter(int n)
{
	struct dvb_adapter *adap;
	struct list_head *entry;
	int found = 0;
	/* We register a bogus adapter here in order to get access to 
	   dvb_adapter_list (if needed) */
	if (! dvb_adapter_list) {
		struct dvb_adapter tmpadap;
		dvb_register_adapter(&tmpadap, "VLOOPBACK tmp", NULL, NULL);
		dvb_adapter_list = tmpadap.list_head.next;
		dvb_unregister_adapter(&tmpadap);
	}

	list_for_each (entry, dvb_adapter_list) {
		adap = list_entry (entry, struct dvb_adapter, list_head);
		if (adap->num == n) {
			found = 1;
			break;
		}
	}
	if (found)
		return adap;
	return NULL;
}
static int vloopback_has_ca(struct dvb_adapter *adap)
{
	struct list_head *entry;

	list_for_each (entry, &adap->device_list) {
		struct dvb_device *dev;
		dev = list_entry (entry, struct dvb_device, list_head);
		if (dev->type == DVB_DEVICE_CA)
			return 1;
	}
	return 0;
}

//static ssize_t read_proc_ca(struct file *filp, char __user *buf, size_t count,
//		loff_t *f_pos)
static int read_proc_ca(char *page, char **start, off_t off, int count,
		int *eof, void *data)
{
	struct vloopback_pipe *vloop = (struct vloopback_pipe *)data;
	printk("reading proc\n");
        count = min(1024, count);
	if (down_interruptible(&vloop->lock))
		return -ERESTARTSYS;
	while(! vloop->data_ready) {
		up(&vloop->lock);
		if (wait_event_interruptible(vloop->wait, vloop->data_ready))
			return -ERESTARTSYS;
		if (down_interruptible(&vloop->lock))
			return -ERESTARTSYS;
	}
	printk("sending proc data :%d\n",count);
	memcpy(page + off, vloop->ioctldata, count);
	*eof = 0;
	vloop->ioctldata[0] = 0;
	vloop->data_ready = 0;
	up(&vloop->lock);
	printk("done\n");
	return count;
}

static int create_pipe(int devn, int idx)
{
	int ret;
	char str[10];

	/* allocate space for this pipe */
	loops[idx]= kmalloc(sizeof(struct vloopback_pipe), GFP_KERNEL);
	if (!loops[idx])
		return -ENOMEM;
	/* set up a new video device plus our private area */
        loops[idx]->adapter= vloopback_dup_adapter(devn);
	if (loops[idx]->adapter == NULL) {
		info("didn't find existing adapter %d", devn);
		return -ENOMEM;
	}
	if (vloopback_has_ca(loops[idx]->adapter)) {
		info("adapter %d already has CA device", devn);
		return -ENOMEM;
	}
	ret = dvb_register_device(loops[idx]->adapter, &loops[idx]->dvbdev,
		&dvbdev_ca, loops[idx], DVB_DEVICE_CA);
	if (ret != 0) {
		info("error registering device adapter%d/ca0",devn);
		return ret;
	}
	sprintf(str, "ca_emu_%d", devn);
	loops[idx]->pe = create_proc_read_entry(str, 0444, NULL,
				read_proc_ca, (void *)loops[idx]);
	if (loops[idx]->pe == NULL) {
		dvb_unregister_device(loops[idx]->dvbdev);
		return -ENOMEM;
	}
	
	loops[idx]->ioctldata=kmalloc(1024, GFP_KERNEL);
	loops[idx]->ioctldata[0] = 0;
	loops[idx]->data_ready = 0;
	init_waitqueue_head(&loops[idx]->wait);
	init_MUTEX(&loops[idx]->lock);
	return 0;
}


/****************************************************************************
 *	init stuff
 ****************************************************************************/


MODULE_AUTHOR("NooneImportant)");
MODULE_DESCRIPTION("DVB CA loopback device.");
module_param_array(devnum,int, &devnum_cnt, S_IRUGO);
MODULE_PARM_DESC(devnum, "Comma-seperated list of adapeter to add CA for");
MODULE_LICENSE("GPL");
MODULE_VERSION( CALOOPBACK_VERSION );

static int __init caloopback_init(void)
{
	int i,ret;
	i=0;
	lb_count=0;
	info("CA loopback driver v"CALOOPBACK_VERSION);

	for (i=0; i < devnum_cnt; i++) {
		ret = create_pipe(devnum[i], lb_count);
		if (ret == 0) {
			info("Loopback %d registered, ca0", devnum[i]);
			lb_count++;
		}else{
			info("Failed to add loopback for adapter %d",devnum[i]);
		}
	}
	if (lb_count == 0)
		return -EFAULT;
	return 0;
}

static void __exit cleanup_caloopback_module(void)
{
	int i;
	info("Unregistering ca loopback devices");
	for (i=0; i < lb_count; i++) {
		remove_proc_entry(loops[i]->pe->name, NULL);
		dvb_unregister_device(loops[i]->dvbdev);
		kfree(loops[i]->ioctldata);
		kfree(loops[i]);
	}
}

module_init(caloopback_init);
module_exit(cleanup_caloopback_module);
